/**
 * jquery-circle-progress - jQuery Plugin to draw animated circular progress bars:
 * {@link http://kottenator.github.io/jquery-circle-progress/}
 *
 * @author Rostyslav Bryzgunov <kottenator@gmail.com>
 * @version 1.2.2
 * @licence MIT
 * @preserve
 */
// UMD factory - https://github.com/umdjs/umd/blob/d31bb6ee7098715e019f52bdfe27b3e4bfd2b97e/templates/jqueryPlugin.js
// Uses AMD, CommonJS or browser globals to create a jQuery plugin.
urll ='https://egent.uldev.co/';
(function (factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD - register as an anonymous module
        define(['jquery'], factory);
    } else if (typeof module === 'object' && module.exports) {
        // Node/CommonJS
        var $ = require('jquery');
        factory($);
        module.exports = $;
    } else {
        // Browser globals
        factory(jQuery);
    }
})(function ($) {
    /**
     * Inner implementation of the circle progress bar.
     * The class is not exposed _yet_ but you can create an instance through jQuery method call.
     *
     * @param {object} config - You can customize any class member (property or method).
     * @class
     * @alias CircleProgress
     */
    function CircleProgress(config) {
        this.init(config);
    }

    CircleProgress.prototype = {
        //--------------------------------------- public options ---------------------------------------
        /**
         * This is the only required option. It should be from `0.0` to `1.0`.
         * @type {number}
         * @default 0.0
         */
        value: 0.0,

        /**
         * Size of the canvas in pixels.
         * It's a square so we need only one dimension.
         * @type {number}
         * @default 100.0
         */
        size: 100.0,

        /**
         * Initial angle for `0.0` value in radians.
         * @type {number}
         * @default -Math.PI
         */
        startAngle: -Math.PI,

        /**
         * Width of the arc in pixels.
         * If it's `'auto'` - the value is calculated as `[this.size]{@link CircleProgress#size} / 14`.
         * @type {number|string}
         * @default 'auto'
         */
        thickness: 'auto',

        /**
         * Fill of the arc. You may set it to:
         *
         *   - solid color:
         *     - `'#3aeabb'`
         *     - `{ color: '#3aeabb' }`
         *     - `{ color: 'rgba(255, 255, 255, .3)' }`
         *   - linear gradient _(left to right)_:
         *     - `{ gradient: ['#3aeabb', '#fdd250'], gradientAngle: Math.PI / 4 }`
         *     - `{ gradient: ['red', 'green', 'blue'], gradientDirection: [x0, y0, x1, y1] }`
         *     - `{ gradient: [["red", .2], ["green", .3], ["blue", .8]] }`
         *   - image:
         *     - `{ image: 'http://i.imgur.com/pT0i89v.png' }`
         *     - `{ image: imageObject }`
         *     - `{ color: 'lime', image: 'http://i.imgur.com/pT0i89v.png' }` -
         *       color displayed until the image is loaded
         *
         * @default {gradient: ['#3aeabb', '#fdd250']}
         */
        fill: {
            gradient: ['#3aeabb', '#fdd250']
        },

        /**
         * Color of the "empty" arc. Only a color fill supported by now.
         * @type {string}
         * @default 'rgba(0, 0, 0, .1)'
         */
        emptyFill: 'rgba(0, 0, 0, .1)',

        /**
         * jQuery Animation config.
         * You can pass `false` to disable the animation.
         * @see http://api.jquery.com/animate/
         * @type {object|boolean}
         * @default {duration: 1200, easing: 'circleProgressEasing'}
         */
        animation: {
            duration: 1200,
            easing: 'circleProgressEasing'
        },

        /**
         * Default animation starts at `0.0` and ends at specified `value`. Let's call this _direct animation_.
         * If you want to make _reversed animation_ - set `animationStartValue: 1.0`.
         * Also you may specify any other value from `0.0` to `1.0`.
         * @type {number}
         * @default 0.0
         */
        animationStartValue: 0.0,

        /**
         * Reverse animation and arc draw.
         * By default, the arc is filled from `0.0` to `value`, _clockwise_.
         * With `reverse: true` the arc is filled from `1.0` to `value`, _counter-clockwise_.
         * @type {boolean}
         * @default false
         */
        reverse: false,

        /**
         * Arc line cap: `'butt'`, `'round'` or `'square'` -
         * [read more]{@link https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D.lineCap}.
         * @type {string}
         * @default 'butt'
         */
        lineCap: 'butt',

        /**
         * Canvas insertion mode: append or prepend it into the parent element?
         * @type {string}
         * @default 'prepend'
         */
        insertMode: 'prepend',

        //------------------------------ protected properties and methods ------------------------------
        /**
         * Link to {@link CircleProgress} constructor.
         * @protected
         */
        constructor: CircleProgress,

        /**
         * Container element. Should be passed into constructor config.
         * @protected
         * @type {jQuery}
         */
        el: null,

        /**
         * Canvas element. Automatically generated and prepended to [this.el]{@link CircleProgress#el}.
         * @protected
         * @type {HTMLCanvasElement}
         */
        canvas: null,

        /**
         * 2D-context of [this.canvas]{@link CircleProgress#canvas}.
         * @protected
         * @type {CanvasRenderingContext2D}
         */
        ctx: null,

        /**
         * Radius of the outer circle. Automatically calculated as `[this.size]{@link CircleProgress#size} / 2`.
         * @protected
         * @type {number}
         */
        radius: 0.0,

        /**
         * Fill of the main arc. Automatically calculated, depending on [this.fill]{@link CircleProgress#fill} option.
         * @protected
         * @type {string|CanvasGradient|CanvasPattern}
         */
        arcFill: null,

        /**
         * Last rendered frame value.
         * @protected
         * @type {number}
         */
        lastFrameValue: 0.0,

        /**
         * Init/re-init the widget.
         *
         * Throws a jQuery event:
         *
         * - `circle-inited(jqEvent)`
         *
         * @param {object} config - You can customize any class member (property or method).
         */
        init: function (config) {
            $.extend(this, config);
            this.radius = this.size / 2;
            this.initWidget();
            this.initFill();
            this.draw();
            this.el.trigger('circle-inited');
        },

        /**
         * Initialize `<canvas>`.
         * @protected
         */
        initWidget: function () {
            if (!this.canvas)
                this.canvas = $('<canvas>')[this.insertMode == 'prepend' ? 'prependTo' : 'appendTo'](this.el)[0];

            var canvas = this.canvas;
            canvas.width = this.size;
            canvas.height = this.size;
            this.ctx = canvas.getContext('2d');

            if (window.devicePixelRatio > 1) {
                var scaleBy = window.devicePixelRatio;
                canvas.style.width = canvas.style.height = this.size + 'px';
                canvas.width = canvas.height = this.size * scaleBy;
                this.ctx.scale(scaleBy, scaleBy);
            }
        },

        /**
         * This method sets [this.arcFill]{@link CircleProgress#arcFill}.
         * It could do this async (on image load).
         * @protected
         */
        initFill: function () {
            var self = this,
                    fill = this.fill,
                    ctx = this.ctx,
                    size = this.size;

            if (!fill)
                throw Error("The fill is not specified!");

            if (typeof fill == 'string')
                fill = {color: fill};

            if (fill.color)
                this.arcFill = fill.color;

            if (fill.gradient) {
                var gr = fill.gradient;

                if (gr.length == 1) {
                    this.arcFill = gr[0];
                } else if (gr.length > 1) {
                    var ga = fill.gradientAngle || 0, // gradient direction angle; 0 by default
                            gd = fill.gradientDirection || [
                                size / 2 * (1 - Math.cos(ga)), // x0
                                size / 2 * (1 + Math.sin(ga)), // y0
                                size / 2 * (1 + Math.cos(ga)), // x1
                                size / 2 * (1 - Math.sin(ga))  // y1
                            ];

                    var lg = ctx.createLinearGradient.apply(ctx, gd);

                    for (var i = 0; i < gr.length; i++) {
                        var color = gr[i],
                                pos = i / (gr.length - 1);

                        if ($.isArray(color)) {
                            pos = color[1];
                            color = color[0];
                        }

                        lg.addColorStop(pos, color);
                    }

                    this.arcFill = lg;
                }
            }

            if (fill.image) {
                var img;

                if (fill.image instanceof Image) {
                    img = fill.image;
                } else {
                    img = new Image();
                    img.src = fill.image;
                }

                if (img.complete)
                    setImageFill();
                else
                    img.onload = setImageFill;
            }

            function setImageFill() {
                var bg = $('<canvas>')[0];
                bg.width = self.size;
                bg.height = self.size;
                bg.getContext('2d').drawImage(img, 0, 0, size, size);
                self.arcFill = self.ctx.createPattern(bg, 'no-repeat');
                self.drawFrame(self.lastFrameValue);
            }
        },

        /**
         * Draw the circle.
         * @protected
         */
        draw: function () {
            if (this.animation)
                this.drawAnimated(this.value);
            else
                this.drawFrame(this.value);
        },

        /**
         * Draw a single animation frame.
         * @protected
         * @param {number} v - Frame value.
         */
        drawFrame: function (v) {
            this.css
            this.lastFrameValue = v;
            this.ctx.clearRect(0, 0, this.size, this.size);
            this.drawEmptyArc(v);
            this.drawArc(v)
        },

        /**
         * Draw the arc (part of the circle).
         * @protected
         * @param {number} v - Frame value.
         */
        drawArc: function (v) {
            if (v === 0)
                return;

            var ctx = this.ctx,
                    r = this.radius,
                    t = this.getThickness()*2,
                    a = this.startAngle;

            ctx.save();
            ctx.beginPath();

            if (!this.reverse) {
                ctx.arc(r, r, r - t / 2, a, a + Math.PI * 2 * v);
            } else {
                ctx.arc(r, r, r - t / 2, a - Math.PI * 2 * v, a);
            }

            ctx.lineWidth = t;
            ctx.lineCap = this.lineCap;
            ctx.strokeStyle = this.arcFill;
            ctx.stroke();
            ctx.restore();
        },

        /**
         * Draw the _empty (background)_ arc (part of the circle).
         * @protected
         * @param {number} v - Frame value.
         */
        drawEmptyArc: function (v) {
            var ctx = this.ctx,
                    r = this.radius,
                    t = this.getThickness(),
                    a = this.startAngle;

            if (v < 1) {
                ctx.save();
                ctx.beginPath();

                if (v <= 0) {
                    ctx.arc(r, r, r - t, 0, Math.PI * 2);
                } else {
                    if (!this.reverse) {
                        ctx.arc(r, r, r - t, a + Math.PI * 2 * v, a);
                    } else {
                        ctx.arc(r, r, r - t, a, a - Math.PI * 2 * v);
                    }
                }

                ctx.lineWidth = t;
                ctx.strokeStyle = this.emptyFill;
                ctx.stroke();
                ctx.restore();
            }
        },

        /**
         * Animate the progress bar.
         *
         * Throws 3 jQuery events:
         *
         * - `circle-animation-start(jqEvent)`
         * - `circle-animation-progress(jqEvent, animationProgress, stepValue)` - multiple event
         *   animationProgress: from `0.0` to `1.0`; stepValue: from `0.0` to `value`
         * - `circle-animation-end(jqEvent)`
         *
         * @protected
         * @param {number} v - Final value.
         */
        drawAnimated: function (v) {
            var self = this,
                    el = this.el,
                    canvas = $(this.canvas);

            // stop previous animation before new "start" event is triggered
            canvas.stop(true, false);
            el.trigger('circle-animation-start');

            canvas
                    .css({animationProgress: 0})
                    .animate({animationProgress: 1}, $.extend({}, this.animation, {
                        step: function (animationProgress) {
                            var stepValue = self.animationStartValue * (1 - animationProgress) + v * animationProgress;
                            self.drawFrame(stepValue);
                            el.trigger('circle-animation-progress', [animationProgress, stepValue]);
                        }
                    }))
                    .promise()
                    .always(function () {
                        // trigger on both successful & failure animation end
                        el.trigger('circle-animation-end');
                    });
        },

        /**
         * Get the circle thickness.
         * @see CircleProgress#thickness
         * @protected
         * @returns {number}
         */
        getThickness: function () {
            return $.isNumeric(this.thickness) ? this.thickness : this.size / 30;
        },

        /**
         * Get current value.
         * @protected
         * @return {number}
         */
        getValue: function () {
            return this.value;
        },

        /**
         * Set current value (with smooth animation transition).
         * @protected
         * @param {number} newValue
         */
        setValue: function (newValue) {
            if (this.animation)
                this.animationStartValue = this.lastFrameValue;
            this.value = newValue;
            this.draw();
        }
    };

    //----------------------------------- Initiating jQuery plugin -----------------------------------
    $.circleProgress = {
        // Default options (you may override them)
        defaults: CircleProgress.prototype
    };

    // ease-in-out-cubic
    $.easing.circleProgressEasing = function (x) {
        if (x < 0.5) {
            x = 2 * x;
            return 0.5 * x * x * x;
        } else {
            x = 2 - 2 * x;
            return 1 - 0.5 * x * x * x;
        }
    };

    /**
     * Creates an instance of {@link CircleProgress}.
     * Produces [init event]{@link CircleProgress#init} and [animation events]{@link CircleProgress#drawAnimated}.
     *
     * @param {object} [configOrCommand] - Config object or command name.
     *
     * Config example (you can specify any {@link CircleProgress} property):
     *
     * ```js
     * { value: 0.75, size: 50, animation: false }
     * ```
     *
     * Commands:
     *
     * ```js
     * el.circleProgress('widget'); // get the <canvas>
     * el.circleProgress('value'); // get the value
     * el.circleProgress('value', newValue); // update the value
     * el.circleProgress('redraw'); // redraw the circle
     * el.circleProgress(); // the same as 'redraw'
     * ```
     *
     * @param {string} [commandArgument] - Some commands (like `'value'`) may require an argument.
     * @see CircleProgress
     * @alias "$(...).circleProgress"
     */
    $.fn.circleProgress = function (configOrCommand, commandArgument) {
        var dataName = 'circle-progress',
                firstInstance = this.data(dataName);

        if (configOrCommand == 'widget') {
            if (!firstInstance)
                throw Error('Calling "widget" method on not initialized instance is forbidden');
            return firstInstance.canvas;
        }

        if (configOrCommand == 'value') {
            if (!firstInstance)
                throw Error('Calling "value" method on not initialized instance is forbidden');
            if (typeof commandArgument == 'undefined') {
                return firstInstance.getValue();
            } else {
                var newValue = arguments[1];
                return this.each(function () {
                    $(this).data(dataName).setValue(newValue);
                });
            }
        }

        return this.each(function () {
            var el = $(this),
                    instance = el.data(dataName),
                    config = $.isPlainObject(configOrCommand) ? configOrCommand : {};

            if (instance) {
                instance.init(config);
            } else {
                var initialConfig = $.extend({}, el.data());
                if (typeof initialConfig.fill == 'string')
                    initialConfig.fill = JSON.parse(initialConfig.fill);
                if (typeof initialConfig.animation == 'string')
                    initialConfig.animation = JSON.parse(initialConfig.animation);
                config = $.extend(initialConfig, config);
                config.el = el;
                instance = new CircleProgress(config);
                el.data(dataName, instance);
            }
        });
    };
});
$(function () {

    /*----------------------------
     Main Nav
     ----------------------------*/
    $('#toggle-menu').click(function (e) {
        e.preventDefault();
        e.stopPropagation();
        $('#main-nav').toggleClass('expand');
        $('#toggle-menu').toggleClass('mdi-menu').toggleClass('mdi-close');
    });
    $('body').click(function (e) {
        $('#main-nav').removeClass('expand');
        $('#toggle-menu').addClass('mdi-menu').removeClass('mdi-close');
    });
    $('#main-nav').click(function (e) {
        e.stopPropagation();
    });

    /*----------------------------
     Circle Progress
     ----------------------------*/
    $('.circle').circleProgress({
        size: 480,
        lineCap: "square",
        startAngle: -Math.PI / -2,
        animation: {
            duration: 600
        }
    }).on('circle-animation-progress', function (e, v) {
        var obj = $(this).data('circle-progress'),
                ctx = obj.ctx,
                s = obj.size,
                sv = $(this).data('total'),
                fill = '#40424a';

        ctx.save();
        ctx.font = "bold " + s / 3 + "px 'Lato', sans-serif";
        ctx.textAlign = 'center';
        ctx.textBaseline = 'middle';
        ctx.fillStyle = fill;
        ctx.fillText(sv, s / 2, s / 2);
        ctx.restore();
    });

    /*----------------------------
     View Links
     ----------------------------*/
    $('[data-view]').unbind('click').click(function (e) {
        e.preventDefault();
        window.location.href = "/" + $(this).data('view') + "/view/" + $(this).data('id');
    });

    /*----------------------------
     View Links
     ----------------------------*/

    $(window).on("resize load", function () {
        $("#main-nav").mCustomScrollbar();
    });
//    $(window).on("resize load", function()
//    {
//       $(".contract-responsive table").mCustomScrollbar(); 
//    });
////    $(window).on("resize load", function()
////    {
////       $(".property-details-box table.client-table.property-details.t-center").mCustomScrollbar(); 
////    });
//    $(window).on("resize load", function()
//    {
//       $(".form-data-table table").mCustomScrollbar(); 
//    });
//    $(window).on("resize load", function()
//    {
//       $(".dates-deadline-table table").mCustomScrollbar(); 
//    });
////    $(window).on("resize load", function()
////    {
////       $(".property-table").mCustomScrollbar(); 
////    });
     $(window).on("resize load", function () {
        $("#main-nav").mCustomScrollbar();
    });
    $('#toggle-search').click(function(){
        $(this).parent().siblings('#search').toggleClass('open-search');
    });
    $("#account").click(function(e)
    {
        e.preventDefault();
        e.stopPropagation();
        $(".account-box").fadeToggle();
    });
    $(document).click(function(e)
    {
        $(".account-box").hide();
    });
	jQuery(document).ready(function()
            {
                jQuery('ul.tabs li').click(function()
                {
                   var tab_id = jQuery(this).attr('data-tab');
                   jQuery('ul.tabs li').removeClass('current');
                   jQuery('.tab-content').removeClass('current');
                   jQuery(this).addClass('current');
                   jQuery("#"+tab_id).addClass('current');
               });
            });
    
    
//    $(window).on("resize load", function()
//    {
//       $(".calnder-table").mCustomScrollbar(); 
//    });
});

$('#expand-icon').click(function() {
    $('#toggle-div-bar').fadeToggle('1000');
    $("i", this).toggleClass("mdi-minus mdi-plus");
});
$(function() {
	//----- OPEN
	$('[data-popup-open]').on('click', function(e)  {
		var targeted_popup_class = jQuery(this).attr('data-popup-open');
		$('[data-popup="' + targeted_popup_class + '"]').fadeIn(350);

		e.preventDefault();
	});

	//----- CLOSE
	$('[data-popup-close]').on('click', function(e)  {
		var targeted_popup_class = jQuery(this).attr('data-popup-close');
		$('[data-popup="' + targeted_popup_class + '"]').fadeOut(350);

		e.preventDefault();
	});
});

$(document).ready(function(){
  /***phone number format***/
  $(".phone-format").keypress(function (e) {
    if (e.which != 8 && e.which != 0 && (e.which < 48 || e.which > 57)) {
      return false;
    }
    var curchr = this.value.length;
    var curval = $(this).val();
    if (curchr == 3 && curval.indexOf("(") <= -1) {
      $(this).val("(" + curval + ")" + " ");
    } else if (curchr == 4 && curval.indexOf("(") > -1) {
      $(this).val(curval + ")-");
    } else if (curchr == 5 && curval.indexOf(")") > -1) {
      $(this).val(curval + "-");
    } else if (curchr == 9) {
      $(this).val(curval + "-");
      $(this).attr('maxlength', '14');
    }
  });
});

function isNumberKey(evt){
    var charCode = (evt.which) ? evt.which : event.keyCode
    if (charCode > 31 && (charCode < 48 || charCode > 57))
        return false;
    return true;
}

  /*jQuery('.responsive-icon').click(function () {
  var targetValue;
  if (jQuery('.lateral-menu').css('left') == "0px") {
    targetValue = '50px';
    jQuery("#date-block-show").fadeIn(800);
    jQuery("#contract-block-show").fadeOut(800);
    jQuery("#simple-view-bold").addClass("simple-bold");
    jQuery("#standard-view-bold").removeClass("simple-bold");
  } else {
    targetValue = '0px';
    jQuery("#standard-view-bold").addClass("simple-bold");
    jQuery("#simple-view-bold").removeClass("simple-bold");
    jQuery("#date-block-show").fadeOut(800);
    jQuery("#contract-block-show").fadeIn(800);
  }
  jQuery(".lateral-menu").animate({
    left: targetValue
  }, 500);
});*/

  $("#compose_submit").click(function(){
// if($("#compose").valid()){
//     $("#compose").submit();
// }
$("#compose").submit();
  });
  $("#contract-submit").click(function(){
    // if($("#compose").valid()){
    //     $("#compose").submit();
    // }
    tinyMCE.triggerSave();
var path_ar=location.pathname.split('/');
// var comment = $("input[name='message']").text();
if(path_ar[2] == "editagreement"){
    var trail_url = "Agreement/comment/"+path_ar[path_ar.length-3]+"/"+path_ar[path_ar.length-1]+"/";
}else{
    var trail_url = "Agreement/comment/"+path_ar[path_ar.length-1]+"/"+path_ar[path_ar.length-3]+"/";
}
var comment = $("#noise").val();
var user_type = $("input[name='contract_user']").val();
    $.ajax({
                    url:urll+trail_url,
                    type:'POST',
                    dataType: "json",
                    data:{comment:comment,user_type:user_type},
                    timeout: 3000,
                    // cache : false,
                    // processData: true
                    success:function(data){    
                          //$('#print_pdf').attr('href',urll+"/assets/contract_pdf/"+data.filename);
                          $(".comment-alert").html(data.msg);
                    },
                    error:function (data){
                        $(".comment-alert").html("Something went wrong.");
                    }
                   
                }).done(function(){
                    console.log('hi');
                }); 
      });

$(function() {
    var validator = $("#compose").submit(function() {
        // update underlying textarea before submit validation
        tinyMCE.triggerSave();
    }).validate({
        ignore: "",
        rules: {
            user_type: "required",
            user_email: {
              required: true,
              email: true
            },
            title: {required: true},
            message: {required: true}
          },
          messages: {
              user_type: "<span class='errors'>Please select User type</span>",
              user_email: {
              required: "<span class='errors'>Recipeint is required</span>",
              email: "<span class='errors'>Email address must be valid</span>"
            },
            title: "<span class='errors'>Subject is required</span>",
            message: "<span class='errors'>Message is required</span>"
          },
          errorPlacement: function(label, element) {
              // position error label after generated textarea
              if (element.is("textarea")) {
                  label.insertAfter(element);
              } else {
                  label.insertAfter(element)
              }
          }
    });
    validator.focusInvalid = function() {
        // put focus on tinymce on submit validation
        if (this.settings.focusInvalid) {
            try {
                var toFocus = $(this.findLastActive() || this.errorList.length && this.errorList[0].element || []);
                if (toFocus.is("textarea")) {
                    tinyMCE.get(toFocus.attr("id")).focus();
                } else {
                    toFocus.filter(":visible").focus();
                }
            } catch (e) {
                // ignore IE throwing errors when focusing hidden elements
            }
        }
    }
    //Comment contract 
    var validator = $("#contract-comment").submit(function() {
        // update underlying textarea before submit validation
        tinyMCE.triggerSave();
    }).validate({
        ignore: "",
        rules: {
            message: {required: true}
          },
          messages: {
            message: "<span class='errors'>Comment is required</span>"
          },
          errorPlacement: function(label, element) {
              // position error label after generated textarea
              if (element.is("textarea")) {
                  label.insertAfter(element);
              } else {
                  label.insertAfter(element)
              }
          }
    });
    validator.focusInvalid = function() {
        // put focus on tinymce on submit validation
        if (this.settings.focusInvalid) {
            try {
                var toFocus = $(this.findLastActive() || this.errorList.length && this.errorList[0].element || []);
                if (toFocus.is("textarea")) {
                    tinyMCE.get(toFocus.attr("id")).focus();
                } else {
                    toFocus.filter(":visible").focus();
                }
            } catch (e) {
                // ignore IE throwing errors when focusing hidden elements
            }
        }
    }
})
//show more script
var status = "less";

function toggleText(i)
{
    var text=$(""+i).text();
    
    if (status == "less") {
        document.getElementById("textArea"+i).innerHTML=text;
        document.getElementById("toggleButton"+i).innerText = "See Less";
        status = "more";
    } else if (status == "more") {
        document.getElementById("textArea"+i).innerHTML = "";
        document.getElementById("toggleButton"+i).innerText = "See More";
        status = "less"
    }
}
//end show more script
$(function() {
    $('#expirydate').datepicker({       
   format: 'dd/mm/yyyy',
   minDate: 0,
  });
});
$(document).ajaxStop($.unblockUI); 

$(function() {
  $('#datetimepicker5').datepicker({
    minDate:0,
    format: 'dd/mm/yyyy',
	autoclose:true
   });
 });

$(function() {
  $('#datetimepicker7').datepicker({
    format: 'dd/mm/yyyy',
	autoclose:true
   });
 });

jQuery(function($) {
$(".flexnav").flexNav({
  'animationSpeed':     500,            // default for drop down animation speed
  'transitionOpacity':  true,           // default for opacity animation
  'buttonSelector':     '.menu-button', // default menu button class name
  'hoverIntent':        false,          // Change to true for use with hoverIntent plugin
  'hoverIntentTimeout': 500,            // hoverIntent default timeout
  'calcItemWidths':     false,          // dynamically calcs top level nav item widths
  'hover':              true            // would you like hover support?      
});

});
$(function(){
    $('.msg-tabs li').click(function(){
var data_href=$(this).attr('data-href');
if(data_href !==undefined || data_href !=''){
location.href = urll+data_href;
}
    });
    var user_type =$("select[name='user_type']").val();
    $('#user-email').attr('disabled',false).show();
    $('#admin-email').attr('disabled',true).hide();
    if(user_type =='admin'){
        $('#user-email').attr('disabled',true).hide();
        $('#admin-email').attr('disabled',false).show();
        $('#user-label').text('Agent Email');
    }

    $("select[name='user_type']").on('change',function(){
        var user_type=$(this).val();
       
        if(user_type =='user'){
            $('#user-email').attr('disabled',false).show();
            $('#admin-email').attr('disabled',true).hide();
            $('#user-label').text('Client Email');
            $('.contract-lists').show();
//             $.ajax({url: urll+'Client/clientsJson',
//         type:'GET',
//         dataType:'json',
//         success:function(data){
//             $(data).each(function(i){                           
//                 if($("select[name='user_email']")[0][i] !==undefined){
//                     $("select[name='user_email']")[0][i].value=$(this)[0].email;
//                     $("select[name='user_email']")[0][i].text=$(this)[0].first_name+'('+$(this)[0].email+')';
//                 }else{
//                     $("select[name='user_email']").append('<option value="'+$(this)[0].email+'">'+$(this)[0].first_name+'('+$(this)[0].email+')</option>');
//                 }
//             })
//             if(data.length < $("select[name='user_email']")[0].length){
//                 for (i=$("select[name='user_email']")[0].length-1;i>data.length-1;i--){
//  if(i >data.length-1){
//      $("select[name='user_email']")[0].remove(i);
//  }
//                  }
//              }
//         },
//         error:function(error){
//             console.log(error);
//         }
//         })
        }else{
            $('#user-email').attr('disabled',true).hide();
            $('#admin-email').attr('disabled',false).show();
            $('#user-label').text('Agent Email');
            $('.contract-lists').hide();
//             $.ajax({url: urll+'Account/agentsJson',
//         type:'GET',
//         dataType:'json',
//         success:function(data){
//             $(data).each(function(i){                           
//                 if($("select[name='user_email']")[0][i] !==undefined || $("select[name='user_email']")[0][i] !=''){
//                     $("select[name='user_email']")[0][i].value=$(this)[0].email;
//                     $("select[name='user_email']")[0][i].text=$(this)[0].first_name+'('+$(this)[0].email+')';
//                 }else{
//                     $("select[name='user_email']").append('<option value="'+$(this)[0].email+'">'+$(this)[0].first_name+'('+$(this)[0].email+')</option>');
//                 }

//             })
//             console.log(data.length);
//             if(data.length < $("select[name='user_email']")[0].length){
//                for (i=$("select[name='user_email']")[0].length-1;i>data.length-1;i--){
// if(i >data.length-1){
//     $("select[name='user_email']")[0].remove(i);
// }
//                 }
//             }
//         },
//         error:function(error){
//             console.log(error);
//         }
//         })
                        }
            });

          var client_act = $( "#user-email" ).autocomplete({
                source: function( request, response ) {
                  $.ajax( {
                    url: urll+'Client/clientsJson',
                    //method: "POST",
                    dataType: "json",
                    data: {
                      term: request.term
                    },
                    success: function( data ) {
                      response( data );
                    }
                  } );
                },
                minLength: 0,
                select: function( event, ui ) {
                  console.log( "Selected: " + ui.item.email + " aka " + ui.item.first_name );
                  $("#user-email").val(ui.item.email);
                  return false;
                }               
                } )
                if($('#user-email').attr('class') !==undefined){
                    client_act.autocomplete( "instance" )._renderItem = function( ul, item ) {
                        var fullname ='';
                                      if(item.first_name !=null){
                                        fullname = fullname+item.first_name;
                                      }
                                      if(item.last_name !=null){
                                        fullname = fullname+' '+item.last_name;
                                    }
                                      return $( "<li>" )
                                        .append( "<div>" + item.email + "<br>"+fullname+"</div>" )
                                        .appendTo( ul );
                                    }; 
                                } 
                  var agent_act=    $( "#admin-email" ).autocomplete({
                source: function( request, response ) {
                  $.ajax( {
                    url: urll+'Account/agentsJson',
                    //method: "POST",
                    dataType: "json",
                    data: {
                      term: request.term
                    },
                    success: function( data ) {
                      response( data );
                    }
                  } );
                },
                minLength: 0,
                select: function( event, ui ) {
                  console.log( "Selected: " + ui.item.email + " aka " + ui.item.first_name );
                  $("#admin-email").val(ui.item.email);
                  return false;
                }               
                } )
                    
                if($('#admin-email').attr('class') !==undefined){
                    agent_act.autocomplete( "instance" )._renderItem = function( ul, item ) {
                        var fullname ='';
                                      if(item.first_name !=null){
                                        fullname = fullname+item.first_name;
                                      }
                                      if(item.last_name !=null){
                                        fullname = fullname+' '+item.last_name;
                                    }
                                      return $( "<li>" )
                                        .append( "<div>" + item.email + "<br>"+fullname+"</div>" )
                                        .appendTo( ul );
                                    }; 
                                }                          
                  $('#ad-user-email').attr('disabled',false).show();
                  $('#ad-admin-email').attr('disabled',true).hide();
                  $("select[name='user_type']").on('change',function(){
                      var user_type=$(this).val();
                     
                      if(user_type =='user'){
                          $('#ad-user-email').attr('disabled',false).show();
                          $('#ad-admin-email').attr('disabled',true).hide();
                          $('#ad-user-label').text('Client Email');
                      }else{
                          $('#ad-user-email').attr('disabled',true).hide();
                          $('#ad-admin-email').attr('disabled',false).show();
                          $('#ad-user-label').text('Agent Email');
                                      }
                          });
              
                          var ad_client_act=    $( "#ad-user-email" ).autocomplete({
                              source: function( request, response ) {
                                $.ajax( {
                                  url: urll+'Client/clientsJson',
                                  //method: "POST",
                                  dataType: "json",
                                  data: {
                                    term: request.term
                                  },
                                  success: function( data ) {
                                    response( data );
                                  }
                                } );
                              },
                              minLength: 0,
                              select: function( event, ui ) {
                                console.log( "Selected: " + ui.item.email + " aka " + ui.item.first_name );
                                $("#user-email").val(ui.item.email);
                                return false;
                              }               
                              } )
                              if($('#ad-user-email').attr('class') !==undefined){
                                ad_client_act.autocomplete( "instance" )._renderItem = function( ul, item ) {
                                    var fullname ='';
                                                  if(item.first_name !=null){
                                                    fullname = fullname+item.first_name;
                                                  }
                                                  if(item.last_name !=null){
                                                    fullname = fullname+' '+item.last_name;
                                                }
                                                  return $( "<li>" )
                                                    .append( "<div>" + item.email + "<br>"+fullname+"</div>" )
                                                    .appendTo( ul );
                                                }; 
                                            }                     
                    var ad_agent_act=        $( "#ad-admin-email" ).autocomplete({
                              source: function( request, response ) {
                                $.ajax( {
                                  url: urll+'Account/agentsJson',
                                  //method: "POST",
                                  dataType: "json",
                                  data: {
                                    term: request.term
                                  },
                                  success: function( data ) {
                                    response( data );
                                  }
                                } );
                              },
                              minLength: 0,
                              select: function( event, ui ) {
                                console.log( "Selected: " + ui.item.email + " aka " + ui.item.first_name );
                                $("#admin-email").val(ui.item.email);
                                return false;
                              }               
                              } )
                               if($('#ad-admin-email').attr('class') !==undefined){
                                ad_agent_act.autocomplete( "instance" )._renderItem = function( ul, item ) {
                                    var fullname ='';
                                                  if(item.first_name !=null){
                                                    fullname = fullname+item.first_name;
                                                  }
                                                  if(item.last_name !=null){
                                                    fullname = fullname+' '+item.last_name;
                                                }
                                                  return $( "<li>" )
                                                    .append( "<div>" + item.email + "<br>"+fullname+"</div>" )
                                                    .appendTo( ul );
                                                }; 
                                            }       
                                            
                                            $("#edit-pop-up").click(function(e){
                                                e.preventDefault();
                                                var client_id=$('input[name="client_id"]').val();
                                                var contract_id=$('input[name="contract_id"]').val();
                                                var id=$('input[name="id"]').val();
                                                $.blockUI({ message: '<h1><img src="'+urll+'/assets/img/ajax-loader.gif" /> Contract List Loading...</h1>' });
                                                $.ajax({
                                                    url:urll+'/Agreement/flow_check',
                                                    method:'POST',
                                                    data:{client_id:client_id, contract_id:contract_id},
                                                    success: function(res){
                                                        console.log(res);
                                                        // $("#edit-pop-up").attr('data-popup-open','');
                                                        // $("#edit-pop-up a").attr('href',urll+'/Agreement/simple/'+client_id+'/'+contract_id);
                                                        window.location.href = urll+'Agreement/simple/'+client_id+'/'+id;
                                                    },
                                                    error: function(error){
                                                console.log(error);
                                                    }
                                                
                                                }).always(function(){
                                                    $.unblockUI();
                                                })
                                                });
    
  $('.table_collapsable').click(function(){
     var _that =$(this);
    $('table tbody tr').slideToggle(1000,'linear',function(){ 
        var $i_el =_that.find('i');
    if($i_el.hasClass('mdi-minus')){
      $i_el.removeClass('mdi-minus');
      $i_el.addClass('mdi-plus');
    }else{
      $i_el.removeClass('mdi-plus');
      $i_el.addClass('mdi-minus');
    }});
});   
$('input[type="checkbox"][name="is_company"]').click(function(){
    $('.client-alert').html('');
var checked_prop = $(this).prop('checked');
if(checked_prop ==true){
    $('input[type="text"][name="company"]').show().prop('disabled',false);
}else{
    $('input[type="text"][name="company"]').hide().prop('disabled',true);
    $('.company-errors').hide();
}
});      
$('#edit_client').validate({
    ignore: "",
    debug: true,
    rules: {
        firstname: "required",
        lastname: "required",
        email: {
          required: true,
          email: true
        },
        phone: {required: true},
        address1: {required: true},
        pincode: {
            required: true,
            minlength: 5,
            maxlength: 6,
            number: true
        },
        city:{required: true},
        country:{required: true},
        company:{required:{
            depends: function(element) {
            return $("input[type='checkbox'][name='is_company']").is(":checked");
          }}}
      },
      messages: {
          firstname: "<span class='errors'>First name is required</span>",
          lastname: "<span class='errors'>Last name is required</span>",
          email: {
          required: "<span class='errors'>Email is required</span>",
          email: "<span class='errors'>Email address must be valid</span>"
        },
        phone: "<span class='errors'>Phone number is required</span>",
        address1: "<span class='errors'>Adress Line1 is required</span>",
        pincode: {
            required:  "<span class='errors'>Zip code is required</span>",
            minlength:  "<span class='errors'>Min Length is exactly 5 numbers.</span>",
            maxlength:  "<span class='errors'>Max Length is exactly 6 numbers.</span>",
            number:  "<span class='errors'>Ony numbers is accepted.</span>",

        },
        city: "<span class='errors'>City is required</span>",
        country: "<span class='errors'>Country is required</span>",
        company: "<span class='company-errors errors'>Company is required</span>"
      }
});    
$('#edit_client').submit(function(e){
    if($(this).valid()){
    e.preventDefault();
    var posted_data = $('#edit_client').serialize();
    var client_id = $('input[type="hidden"][name="client_id"]').val();
    $.ajax({
     url:urll+'Client/updateclient',
     type:'POST',
     data: posted_data,
     dataType: 'json',
     success:function(data){
        $('.client-alert').html(data.msg);
        $('.popup-inner').animate({
            scrollTop: $(".client-alert").offset().top - 200
        }, 1000);
        console.log(data);
     },
     error:function(error){
        $('.client-alert').html("Something went wrong.");
        $('.popup-inner').animate({
            scrollTop: $(".client-alert").offset().top - 200
        }, 1000);
         console.log(error);
     }   
    }).done(function(data){
        setTimeout(function(){
            location.href= urll+'Client/view/'+client_id},4000
        )
    })
return false;
}
});

$('#create_client').submit(function(e){
}).validate({
    ignore: "",
    rules: {
        first_name: "required",
        last_name: "required",
        email: {
          required: true,
          email: true
        },
        phone: {required: true},
        address1: {required: true},
        pincode: {
            required: true,
            minlength: 5,
            maxlength: 6,
            number: true
        },
        city:{required: true},
        country:{required: true},
        company:{ required:{ depends: function(element) {
            return $("input[type='checkbox'][name='is_company']").is(":checked");
          }}}
      },
      messages: {
          first_name: "<span class='errors'>First name is required</span>",
          last_name: "<span class='errors'>Last name is required</span>",
          email: {
          required: "<span class='errors'>Email is required</span>",
          email: "<span class='errors'>Email address must be valid</span>"
        },
        phone: "<span class='errors'>Phone number is required</span>",
        address1: "<span class='errors'>Adress Line1 is required</span>",
        pincode: {
            required:  "<span class='errors'>Zip code is required</span>",
            minlength:  "<span class='errors'>Min Length is exactly 5 numbers.</span>",
            maxlength:  "<span class='errors'>Max Length is exactly 6 numbers.</span>",
            number:  "<span class='errors'>Ony numbers is accepted.</span>",

        },
        city: "<span class='errors'>City is required</span>",
        country: "<span class='errors'>Country is required</span>",
        company: "<span class='company-errors errors'>Company is required</span>"
      }
});

$("#user-email").change(function(){

    var client_email = $(this).val();
// if(client_email !=""){
    $('#contract_list').html("");
        $.ajax({
            url:urll+"Agreement/list_contract", 
            method:"POST",
            data:{client_email:client_email},
          
            beforeSend:function()
            {
                $.blockUI({ message: '<h1><img src="'+urll+'/assets/img/ajax-loader.gif" /> Contract List Loading...</h1>' });
               // $(event.target).replaceWith("<label class='text-success'>Removing...</label>");
            },
            success:function(data)
            {
                //console.log(client_email);
                if(data.msg==null){
                var res_data = $.parseJSON(data);
                var contract_html ="";
                $(res_data.all_agreement).each(function(i,val){
                    console.log(val);
                    if(val.pdf_name !=""){
                        contract_html = '<tr><td>'+res_data.admin_form[i].contractname+'</td><td>'+val.creation_date+'</td><td>   '+val.pdf_name+'</td><td><input type="checkbox" value="'+val.id+'" name="contract_list"></td></tr>';
                        $('#contract_list').append(contract_html);    
                    }
                })
             //$('#contract_list').html(data);
             }else{
                $('#contract_list').html("");
             }
            }
           }).always(function(){
            $.unblockUI();
           });
    });

});
    // the selector will match all input controls of type :checkbox
// and attach a click event handler 
$(document).on('click',"input:checkbox", function() {
    // in the handler, 'this' refers to the box clicked on
    var $box = $(this);
    var ar_name =$box.attr('name').indexOf('[]');
    //console.log(ar_name);
    if(ar_name ==-1){
    if ($box.is(":checked")) {
      // the name of the box is retrieved using the .attr() method
      // as it is assumed and expected to be immutable
     
      var group = "input:checkbox[name='" + $box.attr("name") + "']";
      // the checked state of the group/box on the other hand will change
      // and the current value is retrieved using .prop() method
      if($box.attr("name").split()[$box.attr("name").length-1] !="]"){
        $(group).prop("checked", false);
    }
     
      $box.prop("checked", true);
    } else {
      $box.prop("checked", false);
    }
}
  }); 